# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version 4.2.1-0-g80c4cb6)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc
import wx.dataview

import gettext
_ = gettext.gettext

###########################################################################
## Class MainFrame
###########################################################################

class MainFrame ( wx.Frame ):

    def __init__( self, parent ):
        wx.Frame.__init__ ( self, parent, id = wx.ID_ANY, title = _(u"Motorized Devices Control"), pos = wx.DefaultPosition, size = wx.Size( 800,400 ), style = wx.CAPTION|wx.CLOSE_BOX|wx.RESIZE_BORDER|wx.SYSTEM_MENU|wx.CLIP_CHILDREN|wx.TAB_TRAVERSAL )

        self.SetSizeHints( wx.DefaultSize, wx.DefaultSize )

        self.statusbar = self.CreateStatusBar( 2, wx.STB_DEFAULT_STYLE, wx.ID_ANY )
        self.menu = wx.MenuBar( 0 )
        self.menu_device = wx.Menu()
        self.menu_device_connect = wx.MenuItem( self.menu_device, wx.ID_ANY, _(u"Connect")+ u"\t" + u"Ctrl+K", wx.EmptyString, wx.ITEM_NORMAL )
        self.menu_device.Append( self.menu_device_connect )

        self.menu_device.AppendSeparator()

        self.menu_device_homing = wx.MenuItem( self.menu_device, wx.ID_ANY, _(u"Homing")+ u"\t" + u"Ctrl+Shift+H", wx.EmptyString, wx.ITEM_NORMAL )
        self.menu_device.Append( self.menu_device_homing )

        self.menu_device_increase_magnification_large = wx.MenuItem( self.menu_device, wx.ID_ANY, _(u"Increase Magnification by Large Step")+ u"\t" + u"Ctrl++", wx.EmptyString, wx.ITEM_NORMAL )
        self.menu_device.Append( self.menu_device_increase_magnification_large )

        self.menu_device_decrease_magnification_large = wx.MenuItem( self.menu_device, wx.ID_ANY, _(u"Decrease Magnification by Large Step")+ u"\t" + u"Ctrl+-", wx.EmptyString, wx.ITEM_NORMAL )
        self.menu_device.Append( self.menu_device_decrease_magnification_large )

        self.menu_device_increase_magnification_small = wx.MenuItem( self.menu_device, wx.ID_ANY, _(u"Increase Magnification by Small Step")+ u"\t" + u"Ctrl+Alt++", wx.EmptyString, wx.ITEM_NORMAL )
        self.menu_device.Append( self.menu_device_increase_magnification_small )

        self.menu_device_decrease_magnification_small = wx.MenuItem( self.menu_device, wx.ID_ANY, _(u"Decrease Magnification by Small Step")+ u"\t" + u"Ctrl+Alt+-", wx.EmptyString, wx.ITEM_NORMAL )
        self.menu_device.Append( self.menu_device_decrease_magnification_small )

        self.menu_device_increase_collimation = wx.MenuItem( self.menu_device, wx.ID_ANY, _(u"Increase Collimation")+ u"\t" + u"Ctrl+Shift+=", wx.EmptyString, wx.ITEM_NORMAL )
        self.menu_device.Append( self.menu_device_increase_collimation )

        self.menu_device_decrease_collimation = wx.MenuItem( self.menu_device, wx.ID_ANY, _(u"Decrease Collimation")+ u"\t" + u"Ctrl+Shift+-", wx.EmptyString, wx.ITEM_NORMAL )
        self.menu_device.Append( self.menu_device_decrease_collimation )

        self.menu.Append( self.menu_device, _(u"Device") )

        self.menu_presets = wx.Menu()
        self.menu_presets_add = wx.MenuItem( self.menu_presets, wx.ID_ANY, _(u"Add")+ u"\t" + u"Ctrl+D", wx.EmptyString, wx.ITEM_NORMAL )
        self.menu_presets.Append( self.menu_presets_add )

        self.menu_presets_remove = wx.MenuItem( self.menu_presets, wx.ID_ANY, _(u"Remove"), wx.EmptyString, wx.ITEM_NORMAL )
        self.menu_presets.Append( self.menu_presets_remove )

        self.menu.Append( self.menu_presets, _(u"Presets") )

        self.SetMenuBar( self.menu )


        self.Centre( wx.BOTH )

        # Connect Events
        self.statusbar.Bind( wx.EVT_UPDATE_UI, self.on_statusbar_update_ui )
        self.Bind( wx.EVT_MENU, self.on_menu_device_connect_click, id = self.menu_device_connect.GetId() )
        self.Bind( wx.EVT_UPDATE_UI, self.on_menu_device_connect_update_ui, id = self.menu_device_connect.GetId() )
        self.Bind( wx.EVT_MENU, self.on_menu_device_homing_click, id = self.menu_device_homing.GetId() )
        self.Bind( wx.EVT_UPDATE_UI, self.on_menu_update_ui, id = self.menu_device_homing.GetId() )
        self.Bind( wx.EVT_MENU, self.on_menu_device_increase_magnification_large_selection, id = self.menu_device_increase_magnification_large.GetId() )
        self.Bind( wx.EVT_UPDATE_UI, self.on_menu_update_ui, id = self.menu_device_increase_magnification_large.GetId() )
        self.Bind( wx.EVT_MENU, self.on_menu_device_decrease_magnification_large_selection, id = self.menu_device_decrease_magnification_large.GetId() )
        self.Bind( wx.EVT_UPDATE_UI, self.on_menu_update_ui, id = self.menu_device_decrease_magnification_large.GetId() )
        self.Bind( wx.EVT_MENU, self.on_menu_device_increase_magnification_small_selection, id = self.menu_device_increase_magnification_small.GetId() )
        self.Bind( wx.EVT_UPDATE_UI, self.on_menu_update_ui, id = self.menu_device_increase_magnification_small.GetId() )
        self.Bind( wx.EVT_MENU, self.on_menu_device_decrease_magnification_small_selection, id = self.menu_device_decrease_magnification_small.GetId() )
        self.Bind( wx.EVT_UPDATE_UI, self.on_menu_update_ui, id = self.menu_device_decrease_magnification_small.GetId() )
        self.Bind( wx.EVT_MENU, self.on_menu_device_increase_collimation_selection, id = self.menu_device_increase_collimation.GetId() )
        self.Bind( wx.EVT_UPDATE_UI, self.on_menu_update_ui, id = self.menu_device_increase_collimation.GetId() )
        self.Bind( wx.EVT_MENU, self.on_menu_device_decrease_collimation_selection, id = self.menu_device_decrease_collimation.GetId() )
        self.Bind( wx.EVT_UPDATE_UI, self.on_menu_update_ui, id = self.menu_device_decrease_collimation.GetId() )
        self.Bind( wx.EVT_MENU, self.on_menu_presets_add_selection, id = self.menu_presets_add.GetId() )
        self.Bind( wx.EVT_UPDATE_UI, self.on_menu_update_ui, id = self.menu_presets_add.GetId() )
        self.Bind( wx.EVT_MENU, self.on_menu_presets_remove_selection, id = self.menu_presets_remove.GetId() )
        self.Bind( wx.EVT_UPDATE_UI, self.on_menu_presets_remove_update_ui, id = self.menu_presets_remove.GetId() )

    def __del__( self ):
        pass


    # Virtual event handlers, override them in your derived class
    def on_statusbar_update_ui( self, event ):
        event.Skip()

    def on_menu_device_connect_click( self, event ):
        event.Skip()

    def on_menu_device_connect_update_ui( self, event ):
        event.Skip()

    def on_menu_device_homing_click( self, event ):
        event.Skip()

    def on_menu_update_ui( self, event ):
        event.Skip()

    def on_menu_device_increase_magnification_large_selection( self, event ):
        event.Skip()


    def on_menu_device_decrease_magnification_large_selection( self, event ):
        event.Skip()


    def on_menu_device_increase_magnification_small_selection( self, event ):
        event.Skip()


    def on_menu_device_decrease_magnification_small_selection( self, event ):
        event.Skip()


    def on_menu_device_increase_collimation_selection( self, event ):
        event.Skip()


    def on_menu_device_decrease_collimation_selection( self, event ):
        event.Skip()


    def on_menu_presets_add_selection( self, event ):
        event.Skip()


    def on_menu_presets_remove_selection( self, event ):
        event.Skip()

    def on_menu_presets_remove_update_ui( self, event ):
        event.Skip()


###########################################################################
## Class MainPanel
###########################################################################

class MainPanel ( wx.Panel ):

    def __init__( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( -1,-1 ), style = wx.TAB_TRAVERSAL, name = wx.EmptyString ):
        wx.Panel.__init__ ( self, parent, id = id, pos = pos, size = size, style = style, name = name )

        main_sizer = wx.BoxSizer( wx.VERTICAL )

        sizer_connection = wx.BoxSizer( wx.VERTICAL )

        self.label_serial_port = wx.StaticText( self, wx.ID_ANY, _(u"Serial Port"), wx.DefaultPosition, wx.DefaultSize, 0 )
        self.label_serial_port.Wrap( -1 )

        sizer_connection.Add( self.label_serial_port, 0, wx.ALL, 5 )

        sizer_connection_controls = wx.BoxSizer( wx.HORIZONTAL )

        choice_serial_portChoices = []
        self.choice_serial_port = wx.Choice( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, choice_serial_portChoices, 0 )
        self.choice_serial_port.SetSelection( 0 )
        sizer_connection_controls.Add( self.choice_serial_port, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.button_connect = wx.Button( self, wx.ID_ANY, _(u"Connect"), wx.DefaultPosition, wx.DefaultSize, 0 )
        sizer_connection_controls.Add( self.button_connect, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )


        sizer_connection.Add( sizer_connection_controls, 1, wx.EXPAND, 5 )


        main_sizer.Add( sizer_connection, 0, wx.ALL|wx.EXPAND, 5 )

        device_control_sizer = wx.BoxSizer( wx.HORIZONTAL )

        preset_sizer = wx.BoxSizer( wx.VERTICAL )

        preset_list_sizer = wx.BoxSizer( wx.VERTICAL )

        self.dv_presets = wx.dataview.DataViewListCtrl( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.dataview.DV_ROW_LINES )
        preset_list_sizer.Add( self.dv_presets, 1, wx.ALL|wx.EXPAND, 5 )


        preset_sizer.Add( preset_list_sizer, 1, wx.EXPAND, 5 )

        preset_action_sizer = wx.BoxSizer( wx.HORIZONTAL )

        self.button_preset_add = wx.Button( self, wx.ID_ANY, _(u"Add"), wx.DefaultPosition, wx.DefaultSize, 0 )
        preset_action_sizer.Add( self.button_preset_add, 0, wx.ALL, 5 )


        preset_action_sizer.Add( ( 0, 0), 1, wx.EXPAND, 5 )

        self.button_preset_remove = wx.Button( self, wx.ID_ANY, _(u"Remove"), wx.DefaultPosition, wx.DefaultSize, 0 )
        preset_action_sizer.Add( self.button_preset_remove, 0, wx.ALL, 5 )


        preset_sizer.Add( preset_action_sizer, 0, wx.EXPAND, 5 )


        device_control_sizer.Add( preset_sizer, 0, wx.EXPAND, 5 )

        sizer_device_control = wx.BoxSizer( wx.VERTICAL )

        sizer_wavelength = wx.BoxSizer( wx.VERTICAL )

        self.label_wavelength = wx.StaticText( self, wx.ID_ANY, _(u"Wavelength"), wx.DefaultPosition, wx.DefaultSize, 0 )
        self.label_wavelength.Wrap( -1 )

        sizer_wavelength.Add( self.label_wavelength, 0, wx.ALL, 5 )

        self.label_line_wavelength = wx.StaticLine( self, wx.ID_ANY, wx.DefaultPosition, wx.Size( -1,1 ), wx.LI_HORIZONTAL )
        sizer_wavelength.Add( self.label_line_wavelength, 0, wx.EXPAND |wx.ALL, 5 )

        sizer_wawelength_controls = wx.BoxSizer( wx.HORIZONTAL )

        choice_wavelengthChoices = []
        self.choice_wavelength = wx.Choice( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, choice_wavelengthChoices, 0 )
        self.choice_wavelength.SetSelection( 0 )
        sizer_wawelength_controls.Add( self.choice_wavelength, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )


        sizer_wavelength.Add( sizer_wawelength_controls, 1, wx.ALIGN_CENTER_HORIZONTAL, 5 )


        sizer_device_control.Add( sizer_wavelength, 0, wx.ALL|wx.EXPAND, 5 )

        sizer_magnification = wx.BoxSizer( wx.VERTICAL )

        self.label_magnification = wx.StaticText( self, wx.ID_ANY, _(u"Magnification"), wx.DefaultPosition, wx.DefaultSize, 0 )
        self.label_magnification.Wrap( -1 )

        sizer_magnification.Add( self.label_magnification, 0, wx.ALL, 5 )

        self.label_line_magnification = wx.StaticLine( self, wx.ID_ANY, wx.DefaultPosition, wx.Size( -1,1 ), wx.LI_HORIZONTAL )
        sizer_magnification.Add( self.label_line_magnification, 0, wx.EXPAND |wx.ALL, 5 )

        sizer_magnification_controls = wx.BoxSizer( wx.HORIZONTAL )

        self.button_magnification_decrease_small = wx.Button( self, wx.ID_ANY, _(u"-0.1"), wx.DefaultPosition, wx.Size( 48,-1 ), 0 )
        sizer_magnification_controls.Add( self.button_magnification_decrease_small, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.button_magnification_decrease_large = wx.Button( self, wx.ID_ANY, _(u"-1"), wx.DefaultPosition, wx.Size( 48,-1 ), 0 )
        sizer_magnification_controls.Add( self.button_magnification_decrease_large, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.label_magnification_min = wx.StaticText( self, wx.ID_ANY, _(u"0"), wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_CENTER_HORIZONTAL )
        self.label_magnification_min.Wrap( -1 )

        self.label_magnification_min.SetMinSize( wx.Size( 50,-1 ) )

        sizer_magnification_controls.Add( self.label_magnification_min, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.text_magnification = wx.TextCtrl( self, wx.ID_ANY, _(u"0"), wx.DefaultPosition, wx.DefaultSize, wx.TE_CENTER|wx.TE_PROCESS_ENTER )
        self.text_magnification.SetMinSize( wx.Size( 50,-1 ) )

        sizer_magnification_controls.Add( self.text_magnification, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.label_magnification_max = wx.StaticText( self, wx.ID_ANY, _(u"0"), wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_CENTER_HORIZONTAL )
        self.label_magnification_max.Wrap( -1 )

        self.label_magnification_max.SetMinSize( wx.Size( 50,-1 ) )

        sizer_magnification_controls.Add( self.label_magnification_max, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.button_magnification_increase_large = wx.Button( self, wx.ID_ANY, _(u"+1"), wx.DefaultPosition, wx.Size( 48,-1 ), 0 )
        sizer_magnification_controls.Add( self.button_magnification_increase_large, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.button_magnification_increase_small = wx.Button( self, wx.ID_ANY, _(u"+0.1"), wx.DefaultPosition, wx.Size( 48,-1 ), 0 )
        sizer_magnification_controls.Add( self.button_magnification_increase_small, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )


        sizer_magnification.Add( sizer_magnification_controls, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )


        sizer_device_control.Add( sizer_magnification, 0, wx.ALL|wx.EXPAND, 5 )

        sizer_collimation = wx.BoxSizer( wx.VERTICAL )

        self.label_collimation = wx.StaticText( self, wx.ID_ANY, _(u"Collimation"), wx.DefaultPosition, wx.DefaultSize, 0 )
        self.label_collimation.Wrap( -1 )

        sizer_collimation.Add( self.label_collimation, 0, wx.ALL, 5 )

        self.label_line_collimation = wx.StaticLine( self, wx.ID_ANY, wx.DefaultPosition, wx.Size( -1,1 ), wx.LI_HORIZONTAL )
        sizer_collimation.Add( self.label_line_collimation, 0, wx.EXPAND |wx.ALL, 5 )

        sizer_collimation_controls = wx.BoxSizer( wx.HORIZONTAL )

        self.button_collimation_decrease = wx.Button( self, wx.ID_ANY, _(u"-1"), wx.DefaultPosition, wx.Size( 48,-1 ), 0 )
        sizer_collimation_controls.Add( self.button_collimation_decrease, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.label_collimation_min = wx.StaticText( self, wx.ID_ANY, _(u"0"), wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_CENTER_HORIZONTAL )
        self.label_collimation_min.Wrap( -1 )

        self.label_collimation_min.SetMinSize( wx.Size( 50,-1 ) )

        sizer_collimation_controls.Add( self.label_collimation_min, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.text_collimation = wx.TextCtrl( self, wx.ID_ANY, _(u"0"), wx.DefaultPosition, wx.DefaultSize, wx.TE_CENTER|wx.TE_PROCESS_ENTER )
        self.text_collimation.SetMinSize( wx.Size( 50,-1 ) )

        sizer_collimation_controls.Add( self.text_collimation, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.label_collimation_max = wx.StaticText( self, wx.ID_ANY, _(u"0"), wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_CENTER_HORIZONTAL )
        self.label_collimation_max.Wrap( -1 )

        self.label_collimation_max.SetMinSize( wx.Size( 50,-1 ) )

        sizer_collimation_controls.Add( self.label_collimation_max, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.button_collimation_increase = wx.Button( self, wx.ID_ANY, _(u"+1"), wx.DefaultPosition, wx.Size( 48,-1 ), 0 )
        sizer_collimation_controls.Add( self.button_collimation_increase, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )


        sizer_collimation.Add( sizer_collimation_controls, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )


        sizer_device_control.Add( sizer_collimation, 0, wx.ALL|wx.EXPAND, 5 )


        device_control_sizer.Add( sizer_device_control, 1, wx.EXPAND, 5 )


        main_sizer.Add( device_control_sizer, 1, wx.ALL|wx.EXPAND, 5 )


        self.SetSizer( main_sizer )
        self.Layout()
        main_sizer.Fit( self )
        self.timer = wx.Timer()
        self.timer.SetOwner( self, self.timer.GetId() )
        self.timer.Start( 1000 )


        # Connect Events
        self.choice_serial_port.Bind( wx.EVT_UPDATE_UI, self.on_choice_serial_port_update_ui )
        self.button_connect.Bind( wx.EVT_BUTTON, self.on_connect_click )
        self.button_connect.Bind( wx.EVT_UPDATE_UI, self.on_connect_update_ui )
        self.dv_presets.Bind( wx.dataview.EVT_DATAVIEW_ITEM_ACTIVATED, self.on_presets_item_activated, id = wx.ID_ANY )
        self.dv_presets.Bind( wx.EVT_UPDATE_UI, self.on_presets_update_ui )
        self.button_preset_add.Bind( wx.EVT_BUTTON, self.on_preset_add_click )
        self.button_preset_add.Bind( wx.EVT_UPDATE_UI, self.on_preset_add_update_ui )
        self.button_preset_remove.Bind( wx.EVT_BUTTON, self.on_preset_remove_click )
        self.button_preset_remove.Bind( wx.EVT_UPDATE_UI, self.on_preset_remove_update_ui )
        self.choice_wavelength.Bind( wx.EVT_CHOICE, self.on_wavelength_choice )
        self.choice_wavelength.Bind( wx.EVT_UPDATE_UI, self.on_wavelength_update_ui )
        self.button_magnification_decrease_small.Bind( wx.EVT_BUTTON, self.on_magnification_decrease_small_click )
        self.button_magnification_decrease_small.Bind( wx.EVT_UPDATE_UI, self.on_update_ui )
        self.button_magnification_decrease_large.Bind( wx.EVT_BUTTON, self.on_magnification_decrease_large_click )
        self.button_magnification_decrease_large.Bind( wx.EVT_UPDATE_UI, self.on_update_ui )
        self.label_magnification_min.Bind( wx.EVT_UPDATE_UI, self.on_magnification_min_update_ui )
        self.text_magnification.Bind( wx.EVT_KILL_FOCUS, self.on_magnification_kill_focus )
        self.text_magnification.Bind( wx.EVT_TEXT_ENTER, self.on_magnification_text_enter )
        self.text_magnification.Bind( wx.EVT_UPDATE_UI, self.on_magnification_update_ui )
        self.label_magnification_max.Bind( wx.EVT_UPDATE_UI, self.on_magnification_max_update_ui )
        self.button_magnification_increase_large.Bind( wx.EVT_BUTTON, self.on_magnification_increase_large_click )
        self.button_magnification_increase_large.Bind( wx.EVT_UPDATE_UI, self.on_update_ui )
        self.button_magnification_increase_small.Bind( wx.EVT_BUTTON, self.on_magnification_increase_small_click )
        self.button_magnification_increase_small.Bind( wx.EVT_UPDATE_UI, self.on_update_ui )
        self.button_collimation_decrease.Bind( wx.EVT_BUTTON, self.on_collimation_decrease_click )
        self.button_collimation_decrease.Bind( wx.EVT_UPDATE_UI, self.on_update_ui )
        self.label_collimation_min.Bind( wx.EVT_UPDATE_UI, self.on_collimation_min_update_ui )
        self.text_collimation.Bind( wx.EVT_KILL_FOCUS, self.on_collimation_kill_focus )
        self.text_collimation.Bind( wx.EVT_TEXT_ENTER, self.on_collimation_text_enter )
        self.text_collimation.Bind( wx.EVT_UPDATE_UI, self.on_collimation_update_ui )
        self.label_collimation_max.Bind( wx.EVT_UPDATE_UI, self.on_collimation_max_update_ui )
        self.button_collimation_increase.Bind( wx.EVT_BUTTON, self.on_collimation_increase_click )
        self.button_collimation_increase.Bind( wx.EVT_UPDATE_UI, self.on_update_ui )
        self.Bind( wx.EVT_TIMER, self.on_timer, id=self.timer.GetId() )

    def __del__( self ):
        pass


    # Virtual event handlers, override them in your derived class
    def on_choice_serial_port_update_ui( self, event ):
        event.Skip()

    def on_connect_click( self, event ):
        event.Skip()

    def on_connect_update_ui( self, event ):
        event.Skip()

    def on_presets_item_activated( self, event ):
        event.Skip()

    def on_presets_update_ui( self, event ):
        event.Skip()

    def on_preset_add_click( self, event ):
        event.Skip()

    def on_preset_add_update_ui( self, event ):
        event.Skip()

    def on_preset_remove_click( self, event ):
        event.Skip()

    def on_preset_remove_update_ui( self, event ):
        event.Skip()

    def on_wavelength_choice( self, event ):
        event.Skip()

    def on_wavelength_update_ui( self, event ):
        event.Skip()

    def on_magnification_decrease_small_click( self, event ):
        event.Skip()

    def on_update_ui( self, event ):
        event.Skip()

    def on_magnification_decrease_large_click( self, event ):
        event.Skip()


    def on_magnification_min_update_ui( self, event ):
        event.Skip()

    def on_magnification_kill_focus( self, event ):
        event.Skip()

    def on_magnification_text_enter( self, event ):
        event.Skip()

    def on_magnification_update_ui( self, event ):
        event.Skip()

    def on_magnification_max_update_ui( self, event ):
        event.Skip()

    def on_magnification_increase_large_click( self, event ):
        event.Skip()


    def on_magnification_increase_small_click( self, event ):
        event.Skip()


    def on_collimation_decrease_click( self, event ):
        event.Skip()


    def on_collimation_min_update_ui( self, event ):
        event.Skip()

    def on_collimation_kill_focus( self, event ):
        event.Skip()

    def on_collimation_text_enter( self, event ):
        event.Skip()

    def on_collimation_update_ui( self, event ):
        event.Skip()

    def on_collimation_max_update_ui( self, event ):
        event.Skip()

    def on_collimation_increase_click( self, event ):
        event.Skip()


    def on_timer( self, event ):
        event.Skip()





from __future__ import annotations

import logging
import sys
from pathlib import Path

import wx

from eksma_optics_md_control._gui_base import MainFrame as _MainFrame
from eksma_optics_md_control.main_panel import MainPanel

logger = logging.getLogger(__name__)


class MainFrame(_MainFrame):
    def __init__(self, parent: wx.Frame) -> None:
        super().__init__(parent)

        self._panel = MainPanel(self)

        self.statusbar.SetStatusWidths([200, -1])

        platform_information: wx.PlatformInformation = wx.PlatformInformation.Get()
        if platform_information.GetOperatingSystemId() == wx.OS_WINDOWS_NT:
            if getattr(sys, "frozen", False) and hasattr(sys, "_MEIPASS"):
                icon_path = Path(__file__).resolve().parent.parent / "assets" / "icon.ico"
            else:
                icon_path = Path("assets/icon.ico")

            icon = wx.Icon(str(icon_path), wx.BITMAP_TYPE_ICO)
            self.SetIcon(icon)

    def on_statusbar_update_ui(self, _event: wx.UpdateUIEvent) -> None:
        for i, text in enumerate(self._panel.statusbar_items()):
            self.statusbar.SetStatusText(text, i)

    def on_menu_update_ui(self, event: wx.CommandEvent) -> None:
        self._panel.on_menu_update_ui(event)

    # MARK: Device

    def on_menu_device_connect_click(self, event: wx.CommandEvent) -> None:
        self._panel.on_connect_click(event)

    def on_menu_device_connect_update_ui(self, event: wx.UpdateUIEvent) -> None:
        self._panel.on_connect_update_ui(event)

    def on_menu_device_homing_click(self, event: wx.CommandEvent) -> None:
        self._panel.on_homing_click(event)

    def on_menu_device_increase_magnification_large_selection(self, event: wx.CommandEvent) -> None:
        self._panel.on_magnification_increase_large_click(event)

    def on_menu_device_decrease_magnification_large_selection(self, event: wx.CommandEvent) -> None:
        self._panel.on_magnification_decrease_large_click(event)

    def on_menu_device_increase_magnification_small_selection(self, event: wx.CommandEvent) -> None:
        self._panel.on_magnification_increase_small_click(event)

    def on_menu_device_decrease_magnification_small_selection(self, event: wx.CommandEvent) -> None:
        self._panel.on_magnification_decrease_small_click(event)

    def on_menu_device_increase_collimation_selection(self, event: wx.CommandEvent) -> None:
        self._panel.on_collimation_increase_click(event)

    def on_menu_device_decrease_collimation_selection(self, event: wx.CommandEvent) -> None:
        self._panel.on_collimation_decrease_click(event)

    # MARK: Presets

    def on_menu_presets_add_selection(self, event: wx.CommandEvent) -> None:
        self._panel.on_preset_add_click(event)

    def on_menu_presets_remove_selection(self, event: wx.CommandEvent) -> None:
        self._panel.on_preset_remove_click(event)

    def on_menu_presets_remove_update_ui(self, event: wx.CommandEvent) -> None:
        self._panel.on_preset_remove_update_ui(event)



import logging.config
import os

import wx
import wx.lib.mixins.inspection as wit

from eksma_optics_md_control.main_frame import MainFrame

DEFAULT_LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "standard": {
            "format": "%(asctime)s %(levelname)-8s %(name)s: %(message)s",
        },
    },
    "handlers": {
        "console": {
            "class": "logging.StreamHandler",
            "formatter": "standard",
        },
    },
    "loggers": {
        "": {
            "level": "DEBUG",
            "handlers": ["console"],
        },
    },
}

MD_CONTROL_INSPECTABLE = os.getenv("MD_CONTROL_INSPECTABLE")


class App(wx.App):
    def OnInit(self) -> bool:  # noqa: N802
        frame = MainFrame(None)
        frame.Show()

        self.SetTopWindow(frame)

        return True


class InspectableApp(App, wit.InspectionMixin):
    def OnInit(self) -> bool:  # noqa: N802
        self.Init()

        return super().OnInit()


def main() -> None:
    logging.config.dictConfig(DEFAULT_LOGGING)

    app = InspectableApp() if MD_CONTROL_INSPECTABLE is not None else App()

    app.MainLoop()


if __name__ == "__main__":
    main()



import threading
from collections.abc import Callable
from typing import Any, TypeVar

import wx

T = TypeVar("T")


def schedule_task(
    task: Callable[..., T],
    error: Callable[[Exception], None],
    resume: Callable[[T], None],
    *args: Any,  # noqa: ANN401
    **kwargs: Any,  # noqa: ANN401
) -> None:
    def run() -> None:
        try:
            result = task(*args, **kwargs)
            wx.CallAfter(resume, result)
        except Exception as ex:  # noqa: BLE001
            wx.CallAfter(error, ex)

    threading.Thread(target=run).start()



from __future__ import annotations

import math
import os
import traceback
from dataclasses import dataclass
from enum import Enum
from typing import TYPE_CHECKING, Any

import wx
import wx.dataview
from eksma_optics_motorized_devices.command import IdentificationValue, PresetValue, SystemFlagsValue
from eksma_optics_motorized_devices.control import Control
from eksma_optics_motorized_devices.transport import SerialTransport

from eksma_optics_md_control._gui_base import MainPanel as _MainPanel
from eksma_optics_md_control.task import schedule_task

if TYPE_CHECKING:
    from eksma_optics_motorized_devices.decimal_with_dimension import DecimalWithDimension
    from serial.tools.list_ports_common import ListPortInfo

import logging

if os.name == "nt":
    from eksma_optics_md_control.vendor.serial.tools.list_ports_windows import comports
else:
    from serial.tools.list_ports import comports

logger = logging.getLogger(__name__)


BUTTON_LABEL_CONNECT = "Connect"
BUTTON_LABEL_DISCONNECT = "Disconnect"
MENU_ITEM_LABEL_CONNECT = f"{BUTTON_LABEL_CONNECT}\tCtrl+K"
MENU_ITEM_LABEL_DISCONNECT = f"{BUTTON_LABEL_DISCONNECT}\tCtrl+K"
PRESET_COLUMN_MAGNIFICATION = "Magnification"
PRESET_COLUMN_COLLIMATION = "Collimation"

MAGNIFICATION_DEFAULT = 0
MAGNIFICATION_STEP_LARGE = 1.0
MAGNIFICATION_STEP_SMALL = 0.1

COLLIMATION_DEFAULT = 0
COLLIMATION_STEP = 1

CONTROL_ERROR = "control error"

COLOR_DARK_BLUE = (0x00, 0x42, 0x7A)
COLOR_WHITE = (0xFF, 0xFF, 0xFF)


class ControlError(TypeError):
    pass


class DeviceParametersError(TypeError):
    pass


@dataclass
class DeviceParameters:
    identification: IdentificationValue
    magnification_range: tuple[float, float]
    magnification: float
    collimation_range: tuple[int, int]
    collimation: int
    wavelengths: list[DecimalWithDimension]
    wavelength: DecimalWithDimension
    presets: list[PresetValue]
    preset_remaining: int
    flags: SystemFlagsValue


class State(str, Enum):
    DISCONNECTED = "DISCONNECTED"
    IDLE = "IDLE"
    PROCESSING_HOMING = "PROCESSING_HOMING"
    PROCESSING_MOVING = "PROCESSING_MOVING"


class MainPanel(_MainPanel):
    def __init__(self, parent: wx.Frame) -> None:
        super().__init__(parent)

        self._state = State.DISCONNECTED

        self._serial_ports: list[ListPortInfo] = []
        self._update_serial_ports()

        self._control: Control | None = None
        self._device_parameters: DeviceParameters | None = None

        self.dv_presets.AppendTextColumn(PRESET_COLUMN_MAGNIFICATION, flags=0)
        self.dv_presets.AppendTextColumn(PRESET_COLUMN_COLLIMATION, flags=0)

        self.SetBackgroundColour(COLOR_DARK_BLUE)

        self.label_serial_port.SetForegroundColour(COLOR_WHITE)
        self.label_wavelength.SetForegroundColour(COLOR_WHITE)
        self.label_magnification.SetForegroundColour(COLOR_WHITE)
        self.label_collimation.SetForegroundColour(COLOR_WHITE)

        self.label_line_wavelength.SetBackgroundColour(COLOR_WHITE)
        self.label_line_magnification.SetBackgroundColour(COLOR_WHITE)
        self.label_line_collimation.SetBackgroundColour(COLOR_WHITE)

        self.label_magnification_min.SetForegroundColour(COLOR_WHITE)
        self.label_magnification_max.SetForegroundColour(COLOR_WHITE)
        self.label_collimation_min.SetForegroundColour(COLOR_WHITE)
        self.label_collimation_max.SetForegroundColour(COLOR_WHITE)

        platform_information: wx.PlatformInformation = wx.PlatformInformation.Get()
        if platform_information.GetOperatingSystemId() == wx.OS_WINDOWS_NT:
            font: wx.Font = self.label_serial_port.GetFont()
            font.SetWeight(wx.FONTWEIGHT_SEMIBOLD)

            self.label_serial_port.SetFont(font)
            self.label_wavelength.SetFont(font)
            self.label_magnification.SetFont(font)
            self.label_collimation.SetFont(font)

            self.label_magnification_min.SetFont(font)
            self.label_magnification_max.SetFont(font)
            self.label_collimation_min.SetFont(font)
            self.label_collimation_max.SetFont(font)

    # MARK: Update UI

    def on_menu_update_ui(self, event: wx.UpdateUIEvent) -> None:
        enabled = self._state == State.IDLE
        event.Enable(enabled)

    def on_magnification_min_update_ui(self, event: wx.UpdateUIEvent) -> None:
        enabled = self._state != State.DISCONNECTED
        event.Enable(enabled)

        if self._device_parameters is None:
            event.SetText(str(MAGNIFICATION_DEFAULT))
            return

        event.SetText(str(self._device_parameters.magnification_range[0]))

    def on_magnification_max_update_ui(self, event: wx.UpdateUIEvent) -> None:
        enabled = self._state != State.DISCONNECTED
        event.Enable(enabled)

        if self._device_parameters is None:
            event.SetText(str(MAGNIFICATION_DEFAULT))
            return

        event.SetText(str(self._device_parameters.magnification_range[1]))

    def on_collimation_min_update_ui(self, event: wx.UpdateUIEvent) -> None:
        enabled = self._state != State.DISCONNECTED
        event.Enable(enabled)

        if self._device_parameters is None:
            event.SetText(str(COLLIMATION_DEFAULT))
            return

        event.SetText(str(self._device_parameters.collimation_range[0]))

    def on_collimation_max_update_ui(self, event: wx.UpdateUIEvent) -> None:
        enabled = self._state != State.DISCONNECTED
        event.Enable(enabled)

        if self._device_parameters is None:
            event.SetText(str(COLLIMATION_DEFAULT))
            return

        event.SetText(str(self._device_parameters.collimation_range[1]))

    def on_magnification_update_ui(self, event: wx.UpdateUIEvent) -> None:
        enabled = self._state != State.DISCONNECTED
        event.Enable(enabled)

        if self._device_parameters is None:
            event.SetText(str(COLLIMATION_DEFAULT))
            return

        event_object: wx.TextCtrl = event.GetEventObject()
        if event_object.IsModified():
            return

        event_object.SetValue(str(self._device_parameters.magnification))
        event_object.SetModified(modified=False)

    def on_collimation_update_ui(self, event: wx.UpdateUIEvent) -> None:
        enabled = self._state != State.DISCONNECTED
        event.Enable(enabled)

        if self._device_parameters is None:
            event.SetText(str(COLLIMATION_DEFAULT))
            return

        event_object: wx.TextCtrl = event.GetEventObject()
        if event_object.IsModified():
            return

        event_object.SetValue(str(self._device_parameters.collimation))
        event_object.SetModified(modified=False)

    def on_wavelength_update_ui(self, event: wx.UpdateUIEvent) -> None:
        enabled = self._state != State.DISCONNECTED
        event.Enable(enabled)

        event_object: wx.Choice = event.GetEventObject()

        if self._device_parameters is None:
            event_object.Clear()
            return

        wavelengths = self._device_parameters.wavelengths
        current_value = str(self._device_parameters.wavelength)

        if event_object.GetCount() != len(wavelengths):
            event_object.Clear()
            for wavelength in wavelengths:
                event_object.Append(str(wavelength), wavelength)

        if event_object.GetStringSelection() != current_value:
            event_object.SetStringSelection(current_value)

    def on_presets_update_ui(self, event: wx.UpdateUIEvent) -> None:
        enabled = self._state != State.DISCONNECTED
        event.Enable(enabled)

        # event.GetEventObject() on Windows is broken
        event_object = self.dv_presets
        item_count = event_object.GetItemCount()

        if self._device_parameters is None:
            if item_count > 0:
                event_object.DeleteAllItems()
            return

        presets = self._device_parameters.presets

        needs_refresh = False
        if item_count != len(presets):
            needs_refresh = True
        else:
            for i, preset in enumerate(presets):
                current_magnification = event_object.GetTextValue(i, 0)
                current_collimation = event_object.GetTextValue(i, 1)
                if current_magnification != str(preset.magnification) or current_collimation != str(preset.collimation):
                    needs_refresh = True
                    break

        if needs_refresh:
            event_object.DeleteAllItems()
            for preset in presets:
                event_object.AppendItem((str(preset.magnification), str(preset.collimation)), preset.id)

    def on_preset_add_update_ui(self, event: wx.UpdateUIEvent) -> None:
        enabled = self._state != State.DISCONNECTED

        if self._device_parameters is not None:
            enabled = enabled and self._device_parameters.preset_remaining > 0
        else:
            enabled = False

        event.Enable(enabled)

    def on_preset_remove_update_ui(self, event: wx.UpdateUIEvent) -> None:
        enabled = self._state != State.DISCONNECTED

        if self._device_parameters is not None:
            index = self.dv_presets.GetSelectedRow()
            enabled = index != wx.NOT_FOUND
        else:
            enabled = False

        event.Enable(enabled)

    def on_update_ui(self, event: wx.UpdateUIEvent) -> None:
        enabled = self._state != State.DISCONNECTED
        event.Enable(enabled)

    def on_timer(self, _event: wx.TimerEvent) -> None:
        self._update_serial_ports()

        if self._state in [State.PROCESSING_MOVING, State.PROCESSING_HOMING]:
            return

        if self._device_parameters is None:
            return

        schedule_task(self._update_flags_command_task, self._task_error, self._update_flags_command_resume)

    # MARK: Flags

    def _update_flags_command_task(self) -> SystemFlagsValue:
        if self._control is None:
            raise ControlError

        return self._control.get_flags()

    def _update_flags_command_resume(self, value: SystemFlagsValue) -> None:
        if self._device_parameters is None:
            raise DeviceParametersError

        self._device_parameters.flags = value

        if self._device_parameters.flags & SystemFlagsValue.STATE_CHANGED:
            self._state = State.PROCESSING_MOVING

            schedule_task(self._update_device_parameters_task, self._task_error, self._update_device_parameters_resume)

    # MARK: Connect

    def on_connect_click(self, _event: wx.CommandEvent) -> None:
        if self._state == State.DISCONNECTED:
            self._connect()
        else:
            self._disconnect()

    def on_connect_update_ui(self, event: wx.UpdateUIEvent) -> None:
        connected = self._state != State.DISCONNECTED

        if isinstance(event.GetEventObject(), wx.Menu):
            text = MENU_ITEM_LABEL_DISCONNECT if connected else MENU_ITEM_LABEL_CONNECT
        elif connected:
            text = BUTTON_LABEL_DISCONNECT
        else:
            text = BUTTON_LABEL_CONNECT

        event.SetText(text)

    def _update_serial_ports(self) -> None:
        new_ports = comports()
        current_port: ListPortInfo | None = get_item_client_data(self.choice_serial_port)

        try:
            if all(a == b for a, b in zip(self._serial_ports, new_ports, strict=True)):
                return
        except ValueError:
            pass

        self._serial_ports = new_ports
        self.choice_serial_port.Clear()

        is_connected = False
        for i, port in enumerate(self._serial_ports):
            label = port_info_label(port)
            self.choice_serial_port.Append(label, port)

            if i == 0:
                self.choice_serial_port.SetSelection(0)

            if port == current_port:
                is_connected = True
                self.choice_serial_port.SetSelection(i)

        if not is_connected:
            self._disconnect()

    def _connect(self) -> None:
        self._state = State.PROCESSING_MOVING

        serial_port: ListPortInfo | None = get_item_client_data(self.choice_serial_port)
        if serial_port is None:
            return

        self._control = Control(SerialTransport(serial_port.device))

        schedule_task(self._update_device_parameters_task, self._connect_error, self._update_device_parameters_resume)

    def _connect_error(self, error: Exception) -> None:
        self._state = State.DISCONNECTED

        self._present_error(error)

    def _update_device_parameters_task(self) -> DeviceParameters:
        if self._control is None:
            raise ControlError

        identification = self._control.get_identification()
        magnification_range = self._control.get_magnification_range()
        magnification = self._control.get_magnification()
        collimation_range = self._control.get_collimation_range()
        collimation = self._control.get_collimation()
        wavelengths = self._control.get_wavelengths()
        wavelength = self._control.get_wavelength()
        presets = self._control.get_presets()
        preset_remaining = self._control.get_remaining_presets()
        flags = self._control.get_flags()

        self._control.wait_for_status_idle()

        return DeviceParameters(
            identification=identification,
            magnification_range=magnification_range,
            magnification=magnification,
            collimation_range=collimation_range,
            collimation=collimation,
            wavelengths=wavelengths,
            wavelength=wavelength,
            presets=presets,
            preset_remaining=preset_remaining,
            flags=flags,
        )

    def _update_device_parameters_resume(self, value: DeviceParameters) -> None:
        self._state = State.IDLE

        self._device_parameters = value

    def _disconnect(self) -> None:
        self._state = State.PROCESSING_MOVING

        self._control = None
        self._device_parameters = None

        self._state = State.DISCONNECTED

    # MARK: Homing

    def on_homing_click(self, _event: wx.CommandEvent) -> None:
        if self._state in [State.PROCESSING_MOVING, State.PROCESSING_HOMING]:
            return

        self._state = State.PROCESSING_HOMING

        schedule_task(self._homing_task, self._connect_error, self._homing_resume)

    def _homing_task(self) -> None:
        if self._control is None:
            raise ControlError

        self._control.home()

    def _homing_resume(self, _value: None) -> None:
        self._state = State.IDLE

    # MARK: Magnification

    def on_magnification_decrease_large_click(self, _event: wx.CommandEvent) -> None:
        if self._state in [State.PROCESSING_MOVING, State.PROCESSING_HOMING]:
            return

        if self._device_parameters is None:
            raise DeviceParametersError

        new_mag = round(math.ceil(self._device_parameters.magnification - MAGNIFICATION_STEP_LARGE), 0)
        self._handle_magnification_step_click(new_mag)

    def on_magnification_decrease_small_click(self, _event: wx.CommandEvent) -> None:
        if self._state in [State.PROCESSING_MOVING, State.PROCESSING_HOMING]:
            return

        if self._device_parameters is None:
            raise DeviceParametersError

        new_mag = round(self._device_parameters.magnification - MAGNIFICATION_STEP_SMALL, 1)
        self._handle_magnification_step_click(new_mag)

    def on_magnification_increase_large_click(self, _event: wx.CommandEvent) -> None:
        if self._state in [State.PROCESSING_MOVING, State.PROCESSING_HOMING]:
            return

        if self._device_parameters is None:
            raise DeviceParametersError

        new_mag = round(math.floor(self._device_parameters.magnification + MAGNIFICATION_STEP_LARGE), 0)
        self._handle_magnification_step_click(new_mag)

    def on_magnification_increase_small_click(self, _event: wx.CommandEvent) -> None:
        if self._state in [State.PROCESSING_MOVING, State.PROCESSING_HOMING]:
            return

        if self._device_parameters is None:
            raise DeviceParametersError

        new_mag = round(self._device_parameters.magnification + MAGNIFICATION_STEP_SMALL, 1)
        self._handle_magnification_step_click(new_mag)

    def _handle_magnification_step_click(self, value: float) -> None:
        if self._device_parameters is None:
            raise DeviceParametersError

        self._state = State.PROCESSING_MOVING

        mag_min, mag_max = self._device_parameters.magnification_range
        new_mag = float(min(max(value, mag_min), mag_max))

        self._set_magnification(new_mag)

    def on_magnification_text_enter(self, event: wx.CommandEvent) -> None:
        if self._state in [State.PROCESSING_MOVING, State.PROCESSING_HOMING]:
            return

        self._state = State.PROCESSING_MOVING

        event_object: wx.TextCtrl = event.GetEventObject()

        try:
            value = float(event_object.GetValue())
        except ValueError as ex:
            self._state = State.IDLE
            self._present_error(ex)
            return

        event_object.SetModified(modified=False)

        self._set_magnification(value)

    def on_magnification_kill_focus(self, event: wx.FocusEvent) -> None:
        if self._device_parameters is None:
            raise DeviceParametersError

        event_object: wx.TextCtrl = event.GetEventObject()

        if event_object.IsModified():
            event_object.SetValue(str(self._device_parameters.magnification))
            event_object.SetModified(modified=False)

    def _set_magnification(self, value: float) -> None:
        if self._device_parameters is None:
            raise DeviceParametersError

        col = COLLIMATION_DEFAULT
        self._device_parameters.magnification = value
        self._device_parameters.collimation = col

        schedule_task(
            self._magnification_command_task, self._task_error, self._magnification_command_resume, (value, col)
        )

    def _magnification_command_task(self, value: tuple[float, int]) -> tuple[float, int]:
        if self._control is None:
            raise ControlError

        mag, col = value

        self._control.set_magnification(mag)
        mag = self._control.get_magnification()

        self._control.set_collimation(col)
        col = self._control.get_collimation()

        return mag, col

    def _magnification_command_resume(self, value: tuple[float, int]) -> None:
        if self._device_parameters is None:
            raise DeviceParametersError

        self._state = State.IDLE

        mag, col = value
        self._device_parameters.magnification = mag
        self._device_parameters.collimation = col

    # MARK: Collimation

    def on_collimation_decrease_click(self, _event: wx.CommandEvent) -> None:
        if self._state in [State.PROCESSING_MOVING, State.PROCESSING_HOMING]:
            return

        if self._device_parameters is None:
            raise DeviceParametersError

        new_col = self._device_parameters.collimation - COLLIMATION_STEP
        self._handle_collimation_step_click(new_col)

    def on_collimation_increase_click(self, _event: wx.CommandEvent) -> None:
        if self._state in [State.PROCESSING_MOVING, State.PROCESSING_HOMING]:
            return

        if self._device_parameters is None:
            raise DeviceParametersError

        new_col = self._device_parameters.collimation + COLLIMATION_STEP
        self._handle_collimation_step_click(new_col)

    def _handle_collimation_step_click(self, value: int) -> None:
        if self._device_parameters is None:
            raise DeviceParametersError

        self._state = State.PROCESSING_MOVING

        col_min, col_max = self._device_parameters.collimation_range
        new_col = int(min(max(value, col_min), col_max))

        self._set_collimation(new_col)

    def on_collimation_text_enter(self, event: wx.CommandEvent) -> None:
        if self._state in [State.PROCESSING_MOVING, State.PROCESSING_HOMING]:
            return

        self._state = State.PROCESSING_MOVING

        event_object: wx.TextCtrl = event.GetEventObject()

        try:
            value = int(event_object.GetValue())
        except ValueError as ex:
            self._state = State.IDLE
            self._present_error(ex)
            return

        event_object.SetModified(modified=False)

        self._set_collimation(value)

    def on_collimation_kill_focus(self, event: wx.FocusEvent) -> None:
        if self._device_parameters is None:
            raise DeviceParametersError

        event_object: wx.TextCtrl = event.GetEventObject()

        if event_object.IsModified():
            event_object.SetValue(str(self._device_parameters.collimation))
            event_object.SetModified(modified=False)

    def _set_collimation(self, value: int) -> None:
        if self._device_parameters is None:
            raise DeviceParametersError

        self._device_parameters.collimation = value

        schedule_task(self._collimation_command_task, self._task_error, self._collimation_command_resume, value)

    def _collimation_command_task(self, value: int) -> int:
        if self._control is None:
            raise ControlError

        self._control.set_collimation(value)
        return self._control.get_collimation()

    def _collimation_command_resume(self, value: int) -> None:
        if self._device_parameters is None:
            raise DeviceParametersError

        self._state = State.IDLE

        self._device_parameters.collimation = value

    # MARK: Wavelength

    def on_wavelength_choice(self, _event: wx.CommandEvent) -> None:
        if self._state in [State.PROCESSING_MOVING, State.PROCESSING_HOMING]:
            return

        if self._device_parameters is None:
            raise DeviceParametersError

        wavelength: DecimalWithDimension | None = get_item_client_data(self.choice_wavelength)
        if wavelength is None:
            return

        if wavelength == self._device_parameters.wavelength:
            return

        self._state = State.PROCESSING_MOVING

        schedule_task(self._wavelength_command_task, self._task_error, self._wavelength_command_resume, wavelength)

    def _wavelength_command_task(
        self, value: tuple[DecimalWithDimension, list[PresetValue], int]
    ) -> DecimalWithDimension:
        if self._control is None:
            raise ControlError

        self._control.set_wavelength(int(value))
        wavelength = self._control.get_wavelength()
        presets = self._control.get_presets()
        preset_remaining = self._control.get_remaining_presets()

        return wavelength, presets, preset_remaining

    def _wavelength_command_resume(self, value: tuple[DecimalWithDimension, list[PresetValue], int]) -> None:
        if self._device_parameters is None:
            raise DeviceParametersError

        self._state = State.IDLE

        wavelength, presets, preset_remaining = value
        self._device_parameters.wavelength = wavelength
        self._device_parameters.presets = presets
        self._device_parameters.preset_remaining = preset_remaining

    # MARK: Preset

    def on_preset_add_click(self, _event: wx.CommandEvent) -> None:
        if self._state in [State.PROCESSING_MOVING, State.PROCESSING_HOMING]:
            return

        if self._device_parameters is None:
            raise DeviceParametersError

        self._state = State.PROCESSING_MOVING

        schedule_task(self._add_preset_command_task, self._task_error, self._add_preset_command_resume)

    def _add_preset_command_task(self) -> tuple[list[PresetValue], int]:
        if self._control is None:
            raise ControlError

        self._control.save_preset()
        presets = self._control.get_presets()
        preset_remaining = self._control.get_remaining_presets()

        return presets, preset_remaining

    def _add_preset_command_resume(self, value: tuple[list[PresetValue], int]) -> None:
        if self._device_parameters is None:
            raise DeviceParametersError

        self._state = State.IDLE

        presets, preset_remaining = value
        self._device_parameters.presets = presets
        self._device_parameters.preset_remaining = preset_remaining

    def on_preset_remove_click(self, _event: wx.CommandEvent) -> None:
        if self._state in [State.PROCESSING_MOVING, State.PROCESSING_HOMING]:
            return

        if self._device_parameters is None:
            raise DeviceParametersError

        row = self.dv_presets.GetSelectedRow()
        if row == wx.NOT_FOUND:
            return

        preset_id: int = self.dv_presets.GetItemData(self.dv_presets.RowToItem(row))

        self._state = State.PROCESSING_MOVING

        schedule_task(self._remove_preset_command_task, self._task_error, self._remove_preset_command_resume, preset_id)

    def _remove_preset_command_task(self, value: int) -> tuple[list[PresetValue], int]:
        if self._control is None:
            raise ControlError

        self._control.delete_preset(value)
        presets = self._control.get_presets()
        preset_remaining = self._control.get_remaining_presets()

        return presets, preset_remaining

    def _remove_preset_command_resume(self, value: tuple[list[PresetValue], int]) -> None:
        if self._device_parameters is None:
            raise DeviceParametersError

        self._state = State.IDLE

        presets, preset_remaining = value
        self._device_parameters.presets = presets
        self._device_parameters.preset_remaining = preset_remaining

    def on_presets_item_activated(self, event: wx.dataview.DataViewEvent) -> None:
        if self._state in [State.PROCESSING_MOVING, State.PROCESSING_HOMING]:
            return

        if self._device_parameters is None:
            raise DeviceParametersError

        self._state = State.PROCESSING_MOVING

        event_object: wx.dataview.DataViewListCtrl = event.GetEventObject()

        index = event_object.ItemToRow(event.GetItem())
        preset = self._device_parameters.presets[index]

        self._device_parameters.magnification = preset.magnification
        self._device_parameters.collimation = preset.collimation

        schedule_task(
            self._magnification_command_task,
            self._task_error,
            self._magnification_command_resume,
            (preset.magnification, preset.collimation),
        )

    # MARK: Other

    def statusbar_items(self) -> tuple[str, str]:
        return self._state.value, self._device_description()

    def _device_description(self) -> str:
        if self._device_parameters is None:
            return ""

        return self._device_parameters.identification.description()

    def _task_error(self, error: Exception) -> None:
        self._state = State.IDLE

        self._present_error(error)

    def _present_error(self, error: Exception) -> None:
        try:
            message = traceback.format_exception_only(error)[0].strip()
        except:  # noqa: E722
            message = str(error)

        dialog = wx.MessageDialog(self, message=message, caption="Error", style=wx.OK | wx.ICON_ERROR)
        dialog.ShowModal()
        dialog.Destroy()


def port_info_label(port_info: ListPortInfo) -> str:
    if port_info.description == "n/a":
        return port_info.device

    return f"{port_info.device} ({port_info.description})"


def get_item_client_data(item_container: wx.ItemContainer) -> Any | None:  # noqa: ANN401
    index = item_container.GetSelection()
    if index == wx.NOT_FOUND:
        return None

    return item_container.GetClientData(index)
